"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.junMnemonicToEncryptedWallet = exports.recoverJunKeyProviderFromSeed = exports.slip10DeriveKey = void 0;
const vaulted_key_provider_1 = require("@jolocom/vaulted-key-provider");
const native_core_1 = require("@jolocom/native-core");
const ed25519_hd_key_rn_1 = require("@hawkingnetwork/ed25519-hd-key-rn");
const utils_1 = require("../utils");
const bip39_1 = require("bip39");
const DERIVATION_PATHS = {
    controlKey: "m/73'/0'/0'",
    preRotatedControlKey: "m/73'/0'/1'",
    encKey: "m/73'/1'/0'",
    preRotatedEncKey: "m/73'/1'/1'",
};
const toBase64UrlSafe = (toEncode) => {
    const b64 = toEncode.toString('base64');
    return b64
        .replace(/\+/g, '-')
        .replace(/\//g, '_')
        .replace(/=+$/g, '');
};
const slip10DeriveKey = (seed) => (path) => (0, ed25519_hd_key_rn_1.derivePath)(path, seed).key;
exports.slip10DeriveKey = slip10DeriveKey;
const recoverJunKeyProviderFromSeed = async (seed, newPassword, impl) => {
    const derivationFn = (0, exports.slip10DeriveKey)(seed);
    const controlKey = toBase64UrlSafe(derivationFn(DERIVATION_PATHS.controlKey));
    const preRotatedControlKey = toBase64UrlSafe(derivationFn(DERIVATION_PATHS.preRotatedControlKey));
    const encryptionKey = toBase64UrlSafe(derivationFn(DERIVATION_PATHS.encKey));
    const preRotatedEncryptionKey = toBase64UrlSafe(derivationFn(DERIVATION_PATHS.preRotatedEncKey));
    const { id, encryptedWallet, inceptionEvent } = await (0, native_core_1.getIcpFromKeySet)({
        live_keys: JSON.stringify([controlKey, encryptionKey]),
        pre_rotated_keys: JSON.stringify([
            preRotatedControlKey,
            preRotatedEncryptionKey,
        ]),
        password: newPassword,
    });
    const keyProvider = await vaulted_key_provider_1.SoftwareKeyProvider.newEmptyWallet(impl, id, newPassword);
    keyProvider._encryptedWallet = Buffer.from(encryptedWallet, 'base64');
    return {
        keyProvider,
        inceptionEvent: inceptionEvent,
    };
};
exports.recoverJunKeyProviderFromSeed = recoverJunKeyProviderFromSeed;
const junMnemonicToEncryptedWallet = async (mnemonicPhrase, newPassword, didMethod, impl) => {
    const seed = (0, bip39_1.mnemonicToEntropy)(mnemonicPhrase);
    const { keyProvider, inceptionEvent } = await (0, exports.recoverJunKeyProviderFromSeed)(Buffer.from(seed, 'hex'), newPassword, impl);
    await didMethod.registrar.encounter(inceptionEvent);
    return (0, utils_1.authAsIdentityFromKeyProvider)(keyProvider, newPassword, didMethod.resolver);
};
exports.junMnemonicToEncryptedWallet = junMnemonicToEncryptedWallet;
//# sourceMappingURL=recovery.js.map