"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.mapPublicKeys = exports.publicKeyToAddress = exports.fuelKeyWithEther = exports.keyIdToDid = exports.parseHexOrBase64 = exports.stripHexPrefix = void 0;
const ethereumjs_util_1 = require("ethereumjs-util");
const node_fetch_1 = __importDefault(require("node-fetch"));
const vaulted_key_provider_1 = require("@jolocom/vaulted-key-provider");
const errors_1 = require("../errors");
const utils_1 = require("ethers/lib/utils");
const stripHexPrefix = (hexPrefixedString) => {
    return (0, ethereumjs_util_1.addHexPrefix)(hexPrefixedString).slice(2);
};
exports.stripHexPrefix = stripHexPrefix;
const parseHexOrBase64 = (hexOrB64) => {
    return (0, utils_1.isHexString)((0, ethereumjs_util_1.addHexPrefix)(hexOrB64))
        ? Buffer.from((0, exports.stripHexPrefix)(hexOrB64), 'hex')
        : Buffer.from(hexOrB64, 'base64');
};
exports.parseHexOrBase64 = parseHexOrBase64;
function keyIdToDid(keyId) {
    return keyId.substring(0, keyId.indexOf('#'));
}
exports.keyIdToDid = keyIdToDid;
function fuelKeyWithEther(publicKey) {
    return (0, node_fetch_1.default)('https://faucet.jolocom.com/request/', {
        method: 'POST',
        body: JSON.stringify({ address: (0, exports.publicKeyToAddress)(publicKey) }),
        headers: {
            'Content-Type': 'application/json',
        },
    });
}
exports.fuelKeyWithEther = fuelKeyWithEther;
const publicKeyToAddress = (publicKey) => (0, ethereumjs_util_1.addHexPrefix)((0, ethereumjs_util_1.pubToAddress)(publicKey, true).toString('hex'));
exports.publicKeyToAddress = publicKeyToAddress;
const mapPublicKeys = async (identity, vkpKeys) => {
    const { keyId, did } = identity.didDocument.signer;
    const signingKeyRef = keyId.includes('did:') ? keyId : `${did}${keyId}`;
    const encKey = identity.didDocument.publicKey.find(k => k.type === vaulted_key_provider_1.KeyTypes.x25519KeyAgreementKey2019);
    const encKeyRef = encKey &&
        (encKey.id.startsWith('did:')
            ? encKey.id
            : `${encKey.controller}${encKey.id}`);
    const sigKey = vkpKeys.some(k => k.controller.find(c => c === signingKeyRef));
    if (!sigKey) {
        throw new Error(errors_1.ErrorCodes.PublicKeyNotFound);
    }
    return {
        signingKeyId: signingKeyRef,
        encryptionKeyId: encKeyRef,
    };
};
exports.mapPublicKeys = mapPublicKeys;
//# sourceMappingURL=helper.js.map